from __future__ import division
import math

from game.constants import *

from util import MathTuple
from scenes import TickScene
from game.objects.player import Player, God
from game.objects.gameobjects import Block, Level

from game.objects.various import *
import sys
import data

from pygame_io.events import *

class GameScene(TickScene):
	"""Main game scene

		This scene is the main game. It can transition to the EndScene and the InGameMenu."""
	
	tick_interval = TICK_INTERVAL

	avatar = property(lambda self: self.players[(0,1)[self.avatar_player]].avatar)
	god = property(lambda self: self.players[(1,0)[self.avatar_player]].god)

	def __init__(self, *args, **kwords):
		super(GameScene, self).__init__(*args,**kwords)

		self.environment.screen.clear_color = (50/255,138/255,207/255,0)

		# init game objects
		self.level = Level("test2")

		events = self.environment.events

		self.players = [ Player({'left':events.K_a, 'right':events.K_d, 'up':events.K_w, 'down':events.K_s, 'god1':1,  'god2':3}, self), 
							Player({'left':events.K_LEFT, 'right':events.K_RIGHT, 'up':events.K_UP, 'down':events.K_DOWN, 'god1':1, 'god2':3}, self) ]

		self.players[0].score = 0
		self.players[1].score = 0

		#setup input stuff
		self.input_schemas = [ InputSchema(), InputSchema()]
		self.players[0].avatar.register_with_input_schema(self.input_schemas[0])
		self.players[1].avatar.register_with_input_schema(self.input_schemas[1])
		self.players[0].god.register_with_input_schema(self.input_schemas[1])
		self.players[1].god.register_with_input_schema(self.input_schemas[0])

		self.avatar_player = 1


		self.exit = KbdTap(events.K_ESCAPE)

		self.input_schemas[0].register(self.exit)
		self.input_schemas[1].register(self.exit)



		lvlpos = [int(_.strip()) for _ in self.level.meta['Start'].split(",")]
		self.level_time = int(self.level.meta['Time'])
		self.avatar.position1 = self.avatar.position2 = MathTuple(lvlpos)
		self.players[0].avatar.position1 = self.players[0].avatar.position2 = MathTuple(lvlpos)
		self.players[1].avatar.position1 = self.players[1].avatar.position2 = MathTuple(lvlpos)

		self.moving_objects = [self.avatar]+\
				[Star(position=MathTuple(_)) for _ in self.level.stars_normal]+\
				[TimeStar(position=MathTuple(_)) for _ in self.level.stars_time]+\
				[PointsStar(position=MathTuple(_)) for _ in self.level.stars_points]
		self.static_objects = []

		self.infos = Info(self)

		#play some music
		self.tunes = [ 
				pygame.mixer.Sound(data.filepath(["music", "darktune.ogg"])),
				pygame.mixer.Sound(data.filepath(["music", "lighttune.ogg"]))
			]
		
		self.perish_sound = pygame.mixer.Sound(data.filepath(["music", "max_dies.wav"]))

		self.level_position = MathTuple(0, 0)

		self.init_avatar_run()
		self.tunes[0].play(-1)
		self.tunes[1].play(-1)

	def switch(self):
		print "SWITCH !!!!"
		self.moving_objects.remove(self.avatar)
		self.avatar_player = 1 - self.avatar_player
		self.moving_objects.append(self.avatar)
		self.init_avatar_run()
		self.pause("SWITCH")

	def init_avatar_run(self):
		self.players[self.avatar_player].time_left = self.level_time
		
		new_avatar = self.players[self.avatar_player].avatar
		new_avatar.position1 = new_avatar.position2 = self.players[1-self.avatar_player].avatar.position2
		
		self.tunes[self.avatar_player].set_volume(1.)
		self.tunes[1 - self.avatar_player].set_volume(0.)
		

	def paint(self, phi):
		self.set_screen_position(self.level_position)

		#paint level
		self.level.paint(phi)

		#paint background
		for obj in self.static_objects:
			obj.paint(phi)

		#paint active objects
		for obj in self.moving_objects:
			obj.paint(phi)

		self.god.position = self.environment.screen.translate_coords(self.environment.events.get_mouse_pos())
		self.god.paint(phi)

		self.environment.screen.view_area((0, 0), (80, -60), True) # use negative y to handle lines in a reasonable way; coordinate system is aligned for 10pt font
		self.infos.paint(phi)

		self.set_screen_position(self.level_position) # reset for mouse coordinates. the set_screen_position at the top will not cause additional overhead on opengl!
		

	def tick(self):
		#check for switch and tweak music volumes
		timeleft = self.players[self.avatar_player].time_left
		if timeleft < 0: 
			timeleft = 0

		if self.level_time <= timeleft:
			tunescale = 1
		else:
			tunescale = timeleft / self.level_time

		self.tunes[self.avatar_player].set_volume(tunescale)
		self.tunes[1-self.avatar_player].set_volume(1-tunescale)

		if timeleft == 0:
			self.switch()
		


		self.players[self.avatar_player].time_left -= TICK_INTERVAL / 1000
		self.infos.tick()
		
		#process player commands
		events = self.environment.events

		self.input_schemas[self.avatar_player].process_input(events.get())

		for o in self.moving_objects + self.static_objects + [self.level]:
			o.painter_tick()
			o.tick()
		
		self.god.tick()

		if self.exit.occured():
			sys.exit()


		#update velocity and compute new position2
		for object in self.moving_objects:
			#the previous new position is now our current 
			object.position1 = object.position2

			acceleration = MathTuple(0,0)
			if not object.falling:
				leftcoords = (int(math.floor(object.position1[0])), int(object.position1[1])-1) # the two blocks the player is standing on; FIXME: does not respect collisionpoints
				rightcoords = (int(math.ceil(object.position1[0])), int(object.position1[1])-1)

				gripcoords = (int(round(object.position1[0])), int(object.position1[1])-1)
				acc_factor = self.level.acc_factor_at(gripcoords)
				fric_factor = self.level.fric_factor_at(gripcoords)
			
			if object.falling:
				acc_factor = AIR_ACCELERATION_FACTOR
				fric_factor = AIR_FRICTION_FACTOR
			
			if object.is_heavy:
				acceleration += MathTuple(0, -GRAVITATION)
			
			acceleration += object.get_acceleration_for_tick() * acc_factor
			if object.is_heavy: # typically the same objects
				acceleration -= object.velocity * fric_factor

			object.velocity += acceleration

			object.position2 = object.position1 + object.velocity
			
			#collide and adjust position2
			collision = self.level.collide(object)
			
			if collision[0] != 0:
				object.position2 = MathTuple(object.position2[0] + collision[0], object.position2[1])
				object.velocity = MathTuple(-object.velocity[0] * object.bounce, object.velocity[1])

			if collision[1] != 0:
				object.position2 = MathTuple(object.position2[0], object.position2[1] + collision[1])
				object.velocity = MathTuple(object.velocity[0], -object.velocity[1] * object.bounce)
				if collision[1] > 0:
					object.falling = False
					object.doublejumped = False



		#detect collision between avatar and object
		pl, pr, pu, pd = self.avatar.get_collision_box_boundaries()		

		for object in self.moving_objects:
			if object == self.avatar: continue
			l, r, u, d = object.get_collision_box_boundaries()

			if pl < r and pr > l and pd < u and pu > d:
				object.collides(self.players[self.avatar_player])
			

			

		self.input_schemas[self.avatar_player].clear_after_tick()

		#remove dead objects
		for o in self.moving_objects:
			if o.delete_me:
				self.moving_objects.remove(o)
				
		if self.avatar.position2[1] <= 0:
			self.perish()

		self.move_level()
	
	def move_level(self):
		self.level_position += MathTuple(0.05, 0)
		if self.avatar.position2[0] > (self.level_position[0] + 20):
			self.level_position = MathTuple(self.avatar.position2[0] - 20, self.level_position[1])
		else:
			if self.avatar.position2[0] < self.level_position[0]:
				self.avatar.position2 = MathTuple(self.level_position[0], self.avatar.position2[1])
				collision = self.level.collide(self.avatar)
				if collision != (0, 0):
					self.perish()
		
	def set_screen_position(self, (x, y)):
		self.environment.screen.view_area((x, y + 18.75), (x + 25, y), True)

		if x+25 >= len(self.level.tiles[0]):
			self.end()

	def perish(self):
		self.perish_sound.play()
		self.players[self.avatar_player].time_left -= PERISHTIMEDRAIN
		self.avatar.position2 = self.avatar.position1 = MathTuple(self.level_position + (15, 19))
	
	def pause(self, message="PAUSE"):
		from game.menu import PauseScene
		self.environment.scene = PauseScene(self, message)
	
	def end(self):
		from game.menu import PauseScene, mainmenu
		if self.players[0].score > self.players[1].score:
			message = "Player 0 WINS"
		elif self.players[0].score < self.players[1].score:
			message = "Player 1 WINS"
		else:
			message = "Both players WIN"
		self.environment.scene = PauseScene(self, message, mainmenu(self.environment))
