from objects.opengl import *
from objects.opengl_auxiliary import rectquad, map_to_current_texture
from game.constants import *
import data
from OpenGL.GL import *

class Block(Painter):
	textures={}

	@classmethod
	def paint_transformed(cls, abstract, phi=0):
		try:
			glBindTexture(GL_TEXTURE_2D, cls.textures[abstract.imagefile])
		except KeyError:
			texture = glGenTextures(1)
			cls.textures[abstract.imagefile] = texture
			glBindTexture(GL_TEXTURE_2D, texture)

			map_to_current_texture(["Block","%s.png"%abstract.imagefile])
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)

		glColor(255,255,255,255)
		rectquad((0,1),(1,0),True)

class Level(Painter):
	textures=[]

	@classmethod
	def load_textures(cls):
		tiles = data.load(["Block","tiles.txt"]).readlines()
		for t in tiles:
			t = t.strip()
			if t == "blank":
				cls.textures.append(None)
			else:
				texture = glGenTextures(1)
				glBindTexture(GL_TEXTURE_2D, texture)
				map_to_current_texture(["Block","%s.png"%t])
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)

				cls.textures.append(texture)

	@classmethod
	def paint(cls, abstract, phi=0):
		if not cls.textures: cls.load_textures()

		glColor(0,0,0,0.2)
		rectquad((0,19),(len(abstract.tiles[0]),GOD_MIN_HEIGHT))

		glColor(1,1,1)
		for y,line in enumerate(abstract.tiles):
			for x,tile in enumerate(line):
				if tile:
					try:
						glBindTexture(GL_TEXTURE_2D, cls.textures[tile])
					except IndexError:
						raise Exception("unknown tile")
					rectquad((x,y+1),(x+1,y),True)

class God(StaticSprite3D):
	size = ((-0.5,0.5),(0.5,-0.5))
	imagefile = "god_animated.png"
	depth=2

	@classmethod
	def subset(cls, abstract):
		if abstract.active:
			return [(False, False, 0)]
		else:
			return [(False, False, 1)]

class Player(StaticSprite3D):
	size = ((0,1.9),(1,-0.1)) # painted dimensions don't need to match calculated ones
	imagefile = "max_walking.png"

	depth = 5

	@classmethod
	def subset(cls, abstract):
		if abstract.falling:
			if abstract.velocity[1] > 0: # ascending
				if abstract.velocity[0] < 0:
					return [(True, False, 1)]
				else:
					return [(False, False, 1)]
			else:
				if abstract.velocity[0] < 0:
					return [(True, False, 3)]
				else:
					return [(False, False, 3)]
		else:
			if abstract.go > 0:
				return [(False, False, x) for x in [0,1,2,3]]
			elif abstract.go < 0:
				return [(True, False, x) for x in [0,1,2,3]]
			else:
				return [(False, False, 4)]

class Anvil(StaticSprite3D):
	size = ((0,1),(1,0))
	imagefile = "anvil.png"

	depth = 2

	@classmethod
	def subset(cls, abstract):
		if abstract.falling:
			if abstract.velocity[0]>0:
				return [(False, False, 0)]
			else:
				return [(True, False, 0)]
		else:
			return [(False, False, 1)]

class Lazer(StaticSprite):
	size = ((-0.5,1.5),(1.5,-0.5))
	imagefile = "lazer.png"

class Star(StaticSprite):
	size = ((0,1),(1,0))
	imagefile = "yellowstar.png"

class TimeStar(StaticSprite):
	size = ((0,1),(1,0))
	imagefile = "bluestar.png"

class PointsStar(StaticSprite):
	size = ((0,1),(1,0))
	imagefile = "redstar.png"

class UsedStar(StaticSprite):
	size = ((0,1),(1,0))
	imagefile = "sternschablone.png"
