from __future__ import division
from game.constants import *
from gameobjects import MovableGameObject
import game.objects.opengl
from pygame_io.events import *
from weapons import *
from util import MathTuple
from game.gamescene import TICK_INTERVAL
import data

class Player(object):
	def __init__(self, keys, gamescene):
		self.score = 0
		self.time_left = 0
		self.is_avatar = False
		self.gamescene = gamescene
		self.avatar = Avatar(keys, gamescene)
		self.god = God(keys, gamescene)
		self.hurt_sound = pygame.mixer.Sound(data.filepath(["music", "arr.wav"]))


class Avatar(MovableGameObject):
	def __init__(self, keys, gamescene, *args, **kwords):
		super(Avatar, self).__init__(*args, **kwords)
		self.go = 0
		self.gamescene = gamescene
		self.doublejumped = True
		self.collisionpoints = [MathTuple(x) for x in ((0.04,0),(0.96,0),(0.96,1.6),(0.04,1.6))]

		#init events
		self.left = KbdIsPressed(keys['left'])
		self.right = KbdIsPressed(keys['right'])
		self.jump = KbdTap(keys['up'])
		self.dash_left = KbdSimultaneousTap([keys['down'],keys['left']])
		self.dash_right = KbdSimultaneousTap([keys['down'],keys['right']])

	def register_with_input_schema(self, input_schema):
		input_schema.register(self.left)
		input_schema.register(self.right)
		input_schema.register(self.jump)
		input_schema.register(self.dash_left)
		input_schema.register(self.dash_right)

	def tick(self):
		super(Avatar, self).tick()

		if self.jump.occured():
			if not self.falling:
				self.falling = True
				self.velocity = MathTuple(self.velocity[0], JUMPSPEED1)
				self.doublejumped = False
			elif self.doublejumped == False and self.velocity[1]>=0: # double jump one times, before peak (jumping the golden arches symbol looks strange)
				self.doublejumped = True
				self.velocity = MathTuple(self.velocity[0], JUMPSPEED2)

		self.go = 0
		if self.left.occured(): self.go = -1
		if self.right.occured(): self.go += 1

		if self.dash_left.occured(): print 'dash left'
		if self.dash_right.occured(): print 'dash right'


	def get_acceleration_for_tick(self):
		return MathTuple((SPEED * self.go,0))

	def get_painter(self):
		return game.objects.opengl.Player

class God(MovableGameObject):
	def __init__(self, keys, gamescene, *args, **kwords):
		super(God, self).__init__(*args, **kwords)
		self.sling = MouseDragTrigger(keys['god1'])
		self.weaponchange = MouseTap(keys['god2'])
		self.change_to_anvil = KbdTap(keys['down'])
		self.change_to_lazer = KbdTap(keys['up'])
		self.current_weapon = Anvil
		self.gamescene = gamescene
		self.active = True
		self.currentanvils = ANVILSINAROW
		self.wait = 0
		self.position = MathTuple(0,0)

	def register_with_input_schema(self, input_schema):
		input_schema.register(self.sling)
		input_schema.register(self.weaponchange)
		input_schema.register(self.change_to_anvil)
		input_schema.register(self.change_to_lazer)


	def tick(self):
		super(God, self).tick()
		
		self.wait -= 1
		if self.wait<=0:
			self.active = True
		else:
			self.active = False
		
		if self.change_to_anvil.occured(): self.current_weapon = Anvil
		if self.change_to_lazer.occured(): self.current_weapon = Lazer
		
		
		if self.weaponchange.occurred():
			if self.current_weapon == Anvil:
				self.current_weapon = Lazer
			else:
				self.current_weapon = Anvil

		if self.position[1] < GOD_MIN_HEIGHT:
			self.active = False
		
		if self.current_weapon == Anvil and self.sling.occured() and self.active==True:
			slingstart = MathTuple(self.gamescene.environment.screen.translate_coords(self.sling.start))
			slingend = MathTuple(self.gamescene.environment.screen.translate_coords(self.sling.end))

			weapon = Anvil()
			weapon.position1 = weapon.position2 = slingstart
			if (slingend-slingstart)[0]<0: weapon.velocity = ANVILVELOCITY*(-1,1)
			else: weapon.velocity = ANVILVELOCITY
			
			self.gamescene.moving_objects.append(weapon)
			self.active = False
			self.currentanvils -= 1
			self.wait = ANVILSHORTWAIT
			
			if self.currentanvils ==0 : 
				self.wait=ANVILLONGWAIT
				self.currentanvils=ANVILSINAROW
							
		if self.current_weapon == Lazer and self.sling.occured() and self.active==True:
			slingstart = MathTuple(self.gamescene.environment.screen.translate_coords(self.sling.start))
			slingend = MathTuple(self.gamescene.environment.screen.translate_coords(self.sling.end))

			weapon = Lazer()
			weapon.position1 = weapon.position2 = slingstart
			weapon.velocity = (slingend-slingstart)*LAZERMOD
			vel=abs(weapon.velocity[0])+abs(weapon.velocity[1])
			if vel>LAZERMAXSPEED : 
				wvelocity0 = (weapon.velocity[0]/vel)*LAZERMAXSPEED
				wvelocity1 = (weapon.velocity[1]/vel)*LAZERMAXSPEED
				weapon.velocity=MathTuple(wvelocity0,wvelocity1)
						
			self.gamescene.moving_objects.append(weapon)
			self.active = False
			self.wait = LAZERWAIT

			

	def get_painter(self):
		return game.objects.opengl.God
