from __future__ import division
from game.constants import TICK_INTERVAL
import objects.abstract
from game.objects.gameobjects import MovableGameObject
import game.objects.opengl
from util import MathTuple

class Info(objects.abstract.Paintable):
	def __init__(self, scene):
		self.scene = scene

		self.levelname = objects.abstract.Text()
		self.levelname.position = MathTuple(40, -2)
		self.levelname.align = "center"
		self.levelname.color = (0,0,0)
		self.levelname.size = 3

		self.timeout = objects.abstract.Text()
		self.timeout.position = MathTuple(40, -10)
		self.timeout.text = "0:00"
		self.timeout.align = "center"
		self.timeout.size = 5

		self.player1_label = objects.abstract.Text()
		self.player1_label.text = "Player 1"
		self.player1_label.position = MathTuple(1,-3)
		self.player1_label.color = (0,0,0)
		self.player1_label.size = 2
		self.player1_score = objects.abstract.Text()
		self.player1_score.text = "0"
		self.player1_score.position = MathTuple(1,-6)
		self.player1_score.color = (0,0,0)
		self.player1_score.size = 3

		self.player2_label = objects.abstract.Text()
		self.player2_label.text = "Player 2"
		self.player2_label.position = MathTuple(79,-3)
		self.player2_label.align = "right"
		self.player2_label.size = 2
		self.player2_score = objects.abstract.Text()
		self.player2_score.text = "0"
		self.player2_score.position = MathTuple(79,-6)
		self.player2_score.align = "right"
		self.player2_score.size = 3
	
	def tick(self):
		self.levelname.text = self.scene.level.meta['Title']
		self.player1_score.text = str(self.scene.players[0].score)
		self.player2_score.text = str(self.scene.players[1].score)

		if self.scene.avatar_player == 0: # player 1: black
			self.timeout.color = self.levelname.color = (0,0,0)
		else: # player 2: white
			self.timeout.color = self.levelname.color = (1,1,1)
		self.timeout.text = str(int(self.scene.players[self.scene.avatar_player].time_left))
		# FIXME: update info from self.scene
	
	def paint(self, phi=0):
		self.player1_label.paint(phi)
		self.player1_score.paint(phi)
		self.player2_label.paint(phi)
		self.player2_score.paint(phi)

		self.levelname.paint(phi)
		self.timeout.paint(phi)

class Star(MovableGameObject):
	collisionpoints = [MathTuple(x) for x in ((0,0),(1,0),(1,1),(1,0))]
	addtime = 5
	addpoints = 5
	respawn = 5000 // TICK_INTERVAL
	def __init__(self, position=MathTuple(0,0)):
		super(Star, self).__init__(self) # TypeError: __init__() got multiple values for keyword argument 'position'
		self.position1 = self.position2 = position
		self.used = False
		self.until_respawn = None
		self.bounce = 1
	
	def tick(self):
		super(Star, self).tick()
		if self.used and self.respawn > 0:
			self.until_respawn -= 1
			if self.until_respawn <= 0:
				self.until_respawn = None
				self.used = False

	def collides(self, player):
		if not self.used:
			player.score += self.addpoints
			player.time_left += self.addtime
			self.used = True
			self.until_respawn = self.respawn
	
	def get_painter(self):
		if self.used:
			return game.objects.opengl.UsedStar
		else:
			return game.objects.opengl.Star

class TimeStar(Star):
	addtime = 10
	addpoints = 0
	
	def get_painter(self):
		if self.used:
			return game.objects.opengl.UsedStar
		else:
			return game.objects.opengl.TimeStar

class PointsStar(Star):
	addtime = 0
	addpoints = 10
	
	def get_painter(self):
		if self.used:
			return game.objects.opengl.UsedStar
		else:
			return game.objects.opengl.PointsStar
