from __future__ import division
import objects.opengl
import warnings

class Abstract(object):
	pass

class Paintable(Abstract):
	"""object that can be painted inside a scene"""

	fallback_painter=objects.opengl.Unknown
	def paint(self, *args, **kwords):
		self.get_painter().paint(self, *args, **kwords)
	
	def painter_tick(self, *args, **kwords):
		self.get_painter().tick(*args, **kwords)

	@classmethod
	def get_painter(cls):
		try:
			return getattr(objects.opengl,cls.__name__)
		except AttributeError:
			warnings.warn("no painter found for class %s, using UnknownPainter"%cls.__name__)
			return cls.fallback_painter


class CoordinateSystem(Paintable):
	"""marker to show the current coordinate system (with a red spot in the first quadrant)"""
	def __init__(self, show_terminators=False):
		self.show_terminators=show_terminators


class Positioned(Paintable):
	"""intermediary class to indicate that the object has a (between tick static) position.
	
	you can, at any time, turn an instance into a moving paintable by deleting position and setting position1, position2"""

	def __init__(self):
		self.position = (0,0)

class Moving(Paintable):
	"""intermediary class to indicate that the object has a position (position1 and position2, between which interpolation takes place).
	
	you can, at any time, turn an instance into a positioned paintable by setting position"""

	def __init__(self):
		self.position1 = (0,0)
		self.position2 = (0,0)


class Fnord(Moving): pass # no painter defined; fallback will be used!

class Lenna(Positioned): pass

class Text(Positioned):
	def __init__(self, *args, **kwords):
		super(Text, self).__init__(*args, **kwords)
		self.text=""
		self.font="fixed10"
		self.size=1
		self.color=(255,255,255,255)
		self.align = "left" # of "left", "center", "right"
