"""auxiliary functions for common (frequently debugging) opengl operations"""

from __future__ import division
from OpenGL.GL import *
import pygame.image
import data

def rectquad(topleft, bottomright, texture=False, texture_coords=(1,0,0,1)):
	"""draw a rectangel from topleft to bottomright; set texture coordinates if requested
	
	texxture coordinates are top, left, bottom, right"""

	top=topleft[1]
	left=topleft[0]
	bottom=bottomright[1]
	right=bottomright[0]
	assert left<right
	assert top>bottom

	glBegin(GL_QUADS)

	if texture: glTexCoord2f(texture_coords[1],texture_coords[0])
	glVertex(left, bottom)

	if texture: glTexCoord2f(texture_coords[3],texture_coords[0])
	glVertex(right, bottom)

	if texture: glTexCoord2f(texture_coords[3],texture_coords[2])
	glVertex(right, top)

	if texture: glTexCoord2f(texture_coords[1],texture_coords[2])
	glVertex(left, top)
	glEnd()

def map_to_current_texture(filename):
	"""load a given filename (usable by data.load()) and bind the image data to the current 2d texture
	
	return width and height of image"""
	file =  data.load(filename)
	image = pygame.image.load(file)
	strdata = pygame.image.tostring(image, "RGBA")

	x,y = image.get_size()
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, x, y, 0, GL_RGBA, GL_UNSIGNED_BYTE, strdata)

	return x,y

def map_to_pow2_current_texture(filename):
	"""load a given filename (usable by data.load()), add a border so the image is 2**n x 2**m, and bind to the currend 2d texture

	return ((width, height), (correction coordinates))"""
	file =  data.load(filename)
	image = pygame.image.load(file)
	strdata = pygame.image.tostring(image, "RGBA")

	x,y = image.get_size()
	x2 = y2 = 1
	while x2 < x: x2<<=1
	while y2 < y: y2<<=1

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, x2, y2, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, "\0"*(x2*y2))
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, x, y, GL_RGBA, GL_UNSIGNED_BYTE, strdata)

	return (x,y),(x/x2, y/y2)
