from __future__ import division
from OpenGL.GL import *

class Screen(object):
	def __init__(self, size):
		self.size = size

		glMatrixMode(GL_PROJECTION)

		glEnable(GL_TEXTURE_2D)
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)

		self.current_area=(-1,1),(1,-1)

		self.view_area((-self.aspect,1),(self.aspect,-1))

	def _set_size(self, size):
		self._size=size
		glViewport(0,0,*size)

	size = property(lambda self:self._size, _set_size, None, "the size of the screen in pixels. in opengl, the screen is (on top of the stack) 2 height, with the center in the middle of the screen.")
	aspect = property(lambda self:self.size[0]/self.size[1], None, None, "aspect ratio of the screen")

	def _set_clear_color(self, color):
		self._clear_color=color
		glClearColor(*color)
	clear_color = property(lambda self:self._clear_color, _set_clear_color, None, "background color")

	def view_area(self, (left, top), (right, bottom), accept_distortion=False):
		if ((left,top),(right,bottom))==self.current_area:
			return
		self.current_area=(left,top),(right,bottom)

		width=right-left
		height=top-bottom

		if not accept_distortion:
			if width/height != self.aspect:
				raise Exception("can not show area: would be distorted")

		centerx=(left+right)/2
		centery=(top+bottom)/2

		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		glScalef(2/width, 2/height, 1)
		glTranslate(-centerx,-centery,0)
	
	def translate_coords(self, (x,y)):
		# to opengl identity coordinates / 2
		x = (x-self.size[0]/2) / self.size[0]
		y = (self.size[1]/2-y) / self.size[1]

		# compensate for scale
		(left,top),(right,bottom)=self.current_area
		width=right-left
		height=top-bottom

		x *= width
		y *= height

		# compensate for movement
		centerx=(left+right)/2
		centery=(top+bottom)/2
		
		x += centerx
		y += centery

		return x,y
