"""various stuff, partially to avoid d8ies"""

class MathTuple(tuple):
	"""A tuple you can use in calculations.

	>>> MathTuple(1,1)
	(1, 1)
	>>> MathTuple((2,3))
	(2, 3)
	>>> MathTuple(2,3)+MathTuple(1,5)
	(3, 8)
	>>> MathTuple(2,3)+5
	(7, 8)
	>>> MathTuple(2,3)*MathTuple(1,5)
	(2, 15)
	>>> MathTuple(2,3)*5
	(10, 15)
	>>> MathTuple(2,3)-MathTuple(1,5)
	(1, -2)
	>>> MathTuple(2,3)-5
	(-3, -2)
	"""

	def __new__(cls, *args):
		if len(args) > 1:
			return tuple.__new__(cls, args)
		else:
			return tuple.__new__(cls, args[0])

	def __add__(self, other):
		return self.operator(other, lambda a,b:a+b)

	def __mul__(self, other):
		return self.operator(other, lambda a,b:a*b)

	def __sub__(self, other):
		return self.operator(other, lambda a,b:a-b)

	def operator(self, other, fn):
		if hasattr(other, '__iter__'):
			return MathTuple([fn(a,b) for a,b in zip(self, other)])
		else:
			return MathTuple([fn(a,other) for a in self])

import math
def outer_ints(a, b, skipfirst = False, skiplast = False):
	"""round two floats to a tuple of two ints in matching sequence so the range of floats is included in the range of ints. skip first (a side) and/or last (b side). ,,True,True is inner_ints without first integers.

	>>> outer_ints(0.1, 1.9)
	(0, 2)
	>>> outer_ints(8.1, -2.3)
	(9, -3)
	>>> outer_ints(5, 6)
	(5, 6)
	>>> outer_ints(0.1, 1.9, True)
	(1, 2)
	>>> outer_ints(1.9, 0.1, True)
	(1, 0)
	"""
	if a>b:
		b, a = outer_ints(b, a, skiplast, skipfirst)
		return a, b
	if skipfirst:
		a = int(math.ceil(a))
	else:
		a = int(math.floor(a))
	if skiplast:
		b = int(math.floor(b))
	else:
		b = int(math.ceil(b))
	return a, b

def inner_ints(a, b):
	"""return A, B out of N so [A,B] AND N is [a,b] AND n

	>>> inner_ints(0.4, 3.4)
	(1, 3)
	>>> inner_ints(1, 4)
	(1, 4)
	>>> inner_ints(4, 0.2)
	(4, 1)
	>>> inner_ints(3.2, 3.1)
	"""
	if a>b:
		r = inner_ints(b, a)
		if r:
			return r[1], r[0]
		else:
			return None

	A = int(math.ceil(a))
	B = int(math.floor(b))
	if A>B: return None
	return A, B

def otherrange(a, b):
	"""like range, but different: will return ints in [a, b] instead of [a, b[ and chooses direction automatically
	
	>>> otherrange(1, 5)
	[1, 2, 3, 4, 5]
	>>> otherrange(8, -2)
	[8, 7, 6, 5, 4, 3, 2, 1, 0, -1, -2]"""
	if a>b:
		l=otherrange(b, a)
		l.reverse()
		return l
	return range(a, b+1)

class DefaultList(list):
	"""list that returns a default value for items outside the range

	>>> l = DefaultList("x", ("a", "b", "c"))
	>>> l[0]
	'a'
	>>> l[20]
	'x'
	"""
	def __init__(self, default, items=None):
		for i in items or ():
			self.append(i)
		self.default = default
	def __getitem__(self, item):
		try:
			return super(DefaultList, self).__getitem__(item)
		except IndexError:
			return self.default


def _test():
	import doctest
	doctest.testmod()

if __name__ == "__main__":
	_test();
